#define _CRT_SECURE_NO_WARNINGS
#include "sdl/sdl.h"
#include "smitten.hpp"
#include "smittenUI.hpp"
#include "menutypes.hpp"

#include "LCARSskin.hpp"


#define LINE_SPACING	2
#define LINE_GAP		8
#define LINE_SIZE		32
#define LINE_HEIGHT		(LINE_SIZE+LINE_GAP)

using namespace std;

LCARSskin::LCARSskin(const std::string &visualizer) : SmittenSkin(visualizer) {
	m_MenuArea = sRect(140,315,650,235);
	m_PreviewArea = sRect(446,53,320,200);
	m_StatusArea = sRect(37,12,450,20);
	m_DirectoryArea = sRect(145,563,0,0);

	m_PreviewTextX = 145;
	m_ClockX = 27;
	lastMinuteRendered = -1;

	m_Visualizer = visualizer;
}

std::string &LCARSskin::getSkinName() {
	static std::string n("trek");
	return n;
}

bool LCARSskin::update(float) {
	int hr, min, secs;
	sgxGetTime(hr, min, secs);

	if (min != lastMinuteRendered) {
		lastMinuteRendered = min;
		return true;
	}
	return false;
}

void LCARSskin::drawAlways(MenuHandler *) {
	int y = m_DirectoryArea.y + (LINE_HEIGHT-m_MenuListFont->getHeight())/2;
	
	int hr, min, secs;
	char str[32];

	sgxGetTime(hr, min, secs);
	sprintf(str, "%.2d:%.2d", hr, min);

	m_MenuListFont->drawString(str, m_ClockX, y);
}

void LCARSskin::drawDirectoryPath(MenuHandler *pMenu) {
	int x = m_DirectoryArea.x;
	int y = m_DirectoryArea.y + (LINE_HEIGHT-m_MenuListFont->getHeight())/2;

	for(unsigned int i=1;i<=pMenu->getCurrentMenuListIndex();++i) {
		string txt = pMenu->getDirectedMenuItem(i)->visibleName();
		m_MenuListFont->drawString(txt, x, y);
		x += m_MenuListFont->getWidth(txt);
		if (m_pDirDivider) {
			m_pDirDivider->draw(SmittenUI::get()->getScreenImage(), x, m_DirectoryArea.y + m_pDirDivider->getHeight()/2);
			x += m_pDirDivider->getWidth();
		} else {
			x += 20;
		}
	}
}

void LCARSskin::drawPreview(MenuHandler *pMenu) {
	MenuItem *pCurrent = pMenu->getCurrentMenuItem();
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	if (pCurrent) {
		int y = m_PreviewArea.y;

		m_StatusFont->drawString(pCurrent->visibleName(), m_PreviewTextX, m_PreviewArea.y);
		y += m_StatusFont->getHeight();
		string txt;
		pCurrent->getPreviewText(txt);
		m_StatusFont->drawString(txt, m_PreviewTextX, y);


		SmittenImage *img = pCurrent->getPreviewImage();
		
		if (img) {
			img->draw(pScreenImage, m_PreviewArea.x, m_PreviewArea.y);
		}


	}
}

void LCARSskin::drawStatusInformation(MenuHandler *) {
	MediaHandler *p = SmittenUI::get()->m_pMedia;

	if (p->isPlaying()) {
		m_StatusFont->drawString("Playing...", m_StatusArea.x, m_StatusArea.y);
		m_StatusFont->drawString(p->getCurrentMediaName().c_str(), 150, m_StatusArea.y);
	} else {
		m_StatusFont->drawString("Ready...", m_StatusArea.x, m_StatusArea.y);
	}
}

void LCARSskin::drawCentreSelection(MenuHandler *pMenu) {
	// anything in here will need to appear/be visible to pSkin
	MenuList *pMenuList = pMenu->getCurrentMenuList();
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	int border=0;
	sRect r(m_MenuArea.x+border, m_MenuArea.y+border, m_MenuArea.w-border, m_MenuArea.h-border);
	pScreenImage->clip(r);


	int gap = LINE_GAP;
	int offset = gap + LINE_SIZE;
	int centreLine = m_MenuArea.y + m_MenuArea.h/2 - m_MenuListFont->getHeight()/2;

	// we always have 2 extra items, so we're drawing the ones just off the top,
	// and just off the bottom, when scrolling
	int totalItems = pMenuList->m_List.size();

	int margin = 4;
	int x = m_MenuArea.x+5 - (int)(pMenu->m_PositionX*m_MenuArea.w);
	int y = (centreLine - pMenu->getCurrentMenuItemIndex() * offset) - (int)(pMenu->m_PositionY*offset);

	sRect dr(x-margin, centreLine, m_MenuArea.w-margin, LINE_SIZE);
	if (m_pBar) {
		m_pBar->draw(pScreenImage, dr.x, dr.y);
	} else {
		pScreenImage->fillRect(dr, 51,128,255);
	}

	for(int i=0;i<totalItems;++i) {
		// rely on the ClipRect to prune off-screen items
		drawMenuItem(pMenuList->m_List[i], x, y);
		y += offset;
	}
	pScreenImage->clip();
}

void LCARSskin::drawStatic(MenuHandler *pMenu) {
	// anything in here will need to appear/be visible to pSkin
	MenuList *pMenuList = pMenu->m_EntryList[pMenu->m_CurrentList];
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	int border = 4;
	sRect r(m_MenuArea.x+border, m_MenuArea.y+border, m_MenuArea.w-border, m_MenuArea.h-border);

	if (pMenuList->m_List.size() == 0) {
		return;
	}

	vector<MenuItem *>::iterator it = pMenuList->m_List.begin();

	int margin=LINE_GAP;
	int gap = LINE_GAP;
	int offset = gap + LINE_SIZE;

	int oy = m_MenuArea.y + (m_MenuArea.h-(pMenuList->m_List.size()*offset - gap))/2;
	int x = 10 + m_MenuArea.x - (int)(pMenu->m_PositionX*m_MenuArea.w);
	int y = oy;
	int idx = 0;

	pScreenImage->clip(r);
	
	if (pMenu->m_PositionX == 0) {
		sRect dr(x-margin, oy+pMenu->getCurrentMenuItemIndex()*offset+(int)(pMenu->m_PositionY*offset),m_MenuArea.w-margin,LINE_SIZE);

		if (m_pBar) {
			m_pBar->draw(pScreenImage, dr.x, dr.y-(m_pBar->getHeight()-LINE_SIZE)/2);
		} else {
			pScreenImage->fillRect(dr, 51,128,255);
		}
	}

	for(;it != pMenuList->m_List.end();++it,++idx) {
		drawMenuItem(*it, x, y);
		y += offset;
	}

	pScreenImage->clip();
}


void LCARSskin::drawMenuItem(MenuItem *pItem, int x, int y) {
	sRect dr(x+30,y,32,32);
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	int type = pItem->getIconType();

	if (type < 0 || type >= sizeof(m_pIcons)/sizeof(m_pIcons[0])) {
		// NULL
	} else if (m_pIcons[type]) {
		m_pIcons[type]->draw(pScreenImage, dr.x, dr.y);
	}

	m_MenuListFont->drawString(pItem->visibleName(), x+90, y+((LINE_SIZE+LINE_GAP)-m_MenuListFont->getHeight())/2);

}

void LCARSskin::drawTextFile(sgxTextFile *pText, int line, int , int x, int y) {
	int h = SmittenUI::get()->getScreenImage()->getHeight();
	while(y<h) {
		m_TextBodyFont->drawString(pText->getLine(line), x, y);
		++line;
		y += m_TextBodyFont->getHeight();
		y += LINE_SPACING;
	}
}



