#include "smitten.hpp"
#include "config.hpp"
#include "sgxSystem.hpp"
#include "sgxTextFile.hpp"

using namespace std;

Config *Config::m_pSingleton;

Config *Config::get() {
	if (!m_pSingleton) {
		m_pSingleton = new Config();
	}
	return m_pSingleton;
}

Config::Config() {
	m_AutoPreviewImages = false;
	m_bFullScreen = false;
	m_SlideshowTimer = 7.0f;
	m_SlideshowTimeOut = 6.0f;
	m_ScreensaverTimeout = 200.0f;
	m_bIgnoreHiddenFiles = true;
	m_bEnqueueMediaFiles = false;
	m_SkinName = "std";
	m_VisualizerName = "std";
	m_ScreensaverDirectory = "saver";
}

vector<string> Explode (const string &inString,
                                  const string &separator)
{
   vector<string> returnVector;
   string::size_type start = 0;
   string::size_type end = 0;
 
   while ((end=inString.find (separator, start)) != string::npos)
   {
      returnVector.push_back (inString.substr (start, end-start));
      start = end+separator.size();
   }
 
   returnVector.push_back (inString.substr (start));
 
   return returnVector;
 
}


void Config::parse(const string &filename) {
	sgxTextFile cfg(filename);

	for(int i=0;i<cfg.getLineCount();++i) {
		const string &s = cfg.getLine(i);
		if (s[0] == '#') continue;
		vector<string> line = Explode(s, "|");
		if (line.size() < 2) continue;
		//
		const string &attr = line[0];
		const string &value = line[1];

		if (attr == "autopreview") {
			m_AutoPreviewImages = atoi(value.c_str()) ? true : false;				
		
		} else if (attr == "fullscreen") {
			m_bFullScreen = atoi(value.c_str()) ? true : false;
		
		} else if (attr == "skin") {
			m_SkinName = value;
		} else if (attr == "visualizer") {
			m_VisualizerName = value;

		} else if (attr == "ignoreHiddenFiles") {
			m_bIgnoreHiddenFiles = atoi(value.c_str()) ? true : false;

		} else if (attr == "enqueueMediaFiles") {
			m_bEnqueueMediaFiles = atoi(value.c_str()) ? true : false;
			
		} else if (attr == "screenDir") {
			m_ScreensaverDirectory = value;
		} else if (attr == "screenTimeout") {
			m_ScreensaverTimeout = (float)atof(value.c_str());

		} else if (attr == "slideshowTimer") {
			m_SlideshowTimer = (float)atof(value.c_str());
		} else if (attr == "slideshowTimeOut") {
			m_SlideshowTimeOut = (float)atof(value.c_str());

		} else if (attr == "dir") {
			//	dir lists (dir:name:relative_path) 
			string dir;
			sgxGetWorkingRoot(dir);
			dir += line[2];
			m_DirectoryList.push_back(ConfigDir(value, dir));

		} else if (attr == "smb" || attr == "nfs") {
			m_DirectoryList.push_back(ConfigDir(value, line[2]));

		} else if (attr == "movie_dev") {
			m_MediaCommandList[MEDIA_MOVIE] = value;

		} else if (attr == "music_dev") {
			m_MediaCommandList[MEDIA_MUSIC] = value;
		}
		// Group all media extensions together because the code is too similar
		// TODO: Move m_MediaExtensionList to be a hash?
		int mediaExt = MEDIA_LAST;
		if (attr == "movie_ext")  mediaExt = MEDIA_MOVIE;
		if (attr == "music_ext")  mediaExt = MEDIA_MUSIC;
		if (attr == "photo_ext")  mediaExt = MEDIA_PHOTO;
		if (attr == "text_ext")   mediaExt = MEDIA_TEXT;
		//
		if (mediaExt != MEDIA_LAST) {
			vector<string> ext = Explode(value, ".");
			vector<string>::iterator it = ext.begin();

			for(;it!=ext.end();++it) {
				m_MediaExtensionList[mediaExt].push_back(*it);
			}
		}


	}
}
