/*
    Smitten: A media centre application, for playing movies, music, picture
             slideshows, and reading text on your TV or computer.

    Copyright (C) 2008 Steven Goodwin

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; 
    version 2.0.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
#include <windows.h>
#include "SDL/SDL.h" 
#include "smitten.hpp"
#include "smittenUI.hpp"
#include "sgxSystem.hpp"

/*
Smitten is a media centre application, for playing movies, music, picture
slideshows, and reading text on your TV or computer.

Smitten differs from traditional media centres in three major ways:

1. It contains no media playback at all! This makes it very small (and
   suitable for porting to embedded systems) and quick to initialize.

2. It targets a TV display and interface. This means there's less information 
   on any particular screen, and a much simpler control interface consisting
   of left, right, up, down, and select. No mouse required!

3. It is also highly skinnable, and cross-platform to most systems.

Essentially, Smitten provides an easy-to-use menu system that allows the user
to navigate a hierarchy of files, and view them using an (external) tool.

*/

void SmittenInit(SDL_Surface *pScreen);
bool SmittenUpdate(SDL_Event *pEvent, int telapsMilliSeconds);
bool SmittenPaint();
	
void init(SDL_Surface *pScreen) {
	SmittenInit(pScreen);
}

bool update(SDL_Event *pEvent, int telapsMilliSeconds) {
	if (pEvent->type == SDL_QUIT) {
		return false;
	}
	return SmittenUpdate(pEvent, telapsMilliSeconds);	// continue running
}

bool paint() {
	return SmittenPaint();// flip surfaces beause we drew something
}

int SDL_main( int , char* [] ) { 

	int width = 800;
	int height = 600;

	Config *pConfig = Config::get();
	pConfig->parse("config/main.cfg");

	SmittenUI::get();

	width = SmittenUI::get()->m_pSkin->getWidth();
	height = SmittenUI::get()->m_pSkin->getHeight();

	SDL_Init(SDL_INIT_EVERYTHING); 
	SDL_WM_SetCaption("Smitten", NULL);

	SDL_Surface *pScreen = SDL_SetVideoMode(width, height, 16, (pConfig->m_bFullScreen?SDL_FULLSCREEN:0) | SDL_HWSURFACE | SDL_DOUBLEBUF);

	if (pScreen == NULL) {
		SDL_Quit();
		return -1;
	}

	SDL_Event ev;
	Uint32 prev_time, curr_time;
	Uint32 period  = 1000/60; /* 16 milliseconds between frames */
	Uint32 delta_time = 0;
	bool bIsRunning = true;

	init(pScreen);

	prev_time = SDL_GetTicks();

	while(SDL_PollEvent(&ev) >= 0 && bIsRunning) {
		// We place this at the top so that we ensure
		// the first 'dirty' framebuffer is drawn.
		if (paint()) {
			SDL_Flip(pScreen);
		}
		bIsRunning = update(&ev, delta_time);
		do {
			curr_time = SDL_GetTicks();
			/* Check for 49 day wrap-around */
			if (curr_time > prev_time)
				delta_time = curr_time-prev_time;
			else
				delta_time = 0xffffffff - (prev_time-curr_time) + 1;

			int timeLeft = period - delta_time;
			if (timeLeft < 0) {
				timeLeft = 1;
			}
			SDL_Delay(timeLeft);
		} while(SDL_PollEvent(&ev) >= 0 && delta_time < period);

		prev_time = curr_time;
	}

	SDL_FreeSurface(pScreen);

	SDL_Quit(); 

	return 0; 
}
