#include "smitten.hpp"
#include "sgxSystem.hpp"

using namespace std;

MediaHandler::MediaHandler() {
	m_ProcessId = 0;
	m_bPlayingMultipleTracks = false;
}

void MediaHandler::update(float ) {
}

bool MediaHandler::start(int filterType, const vector<std::string> &filelist) {

	string fullCommand = Config::get()->m_MediaCommandList[filterType];
	if (fullCommand == "") {
		return false;
	}

	m_Current = m_CurrentName = "Multiple files";
	m_MediaType = filterType;
	m_bPlayingMultipleTracks = true;

	string argList = ""; 
	std::vector<std::string>::const_iterator it = filelist.begin();
	for(;it!=filelist.end();++it) {
		argList += "\"";
		argList += (*it);
		argList += "\" ";
	}

	int fn = fullCommand.find("%s");
	fullCommand.replace(fn,fn+1, argList);

	m_ProcessId = sgxSystem(fullCommand);
	if (m_ProcessId) {
		return true;		
	}

	return false;
}

bool MediaHandler::start(int type, const std::string &filename) {

	string fullCommand = Config::get()->m_MediaCommandList[type];
	if (fullCommand == "") {
		return false;
	}

	m_Current = filename;
	m_MediaType = type;
	m_bPlayingMultipleTracks = false;

	m_CurrentName = filename;
	int ls = filename.rfind("/");
	if (ls == -1) {
		ls = filename.rfind("\\");
	}
	if (ls != -1) {
		m_CurrentName = filename.substr(ls+1);
	}

	string quoted("\"");
	quoted += filename;
	quoted += "\"";

	int fn = fullCommand.find("%s");
	fullCommand.replace(fn,fn+1, quoted);

	m_ProcessId = sgxSystem(fullCommand);
	if (m_ProcessId) {
		return true;		
	}

	return false;
}

bool MediaHandler::pause() {
	return false;
}

bool MediaHandler::stop() {
	if (isPlaying()) {
		sgxExitProcess(m_ProcessId);
		m_ProcessId = 0;
		return true;
	}
	return false;
}

bool MediaHandler::isPlaying() const {
	if (m_ProcessId) {
		if (!sgxIsProcessRunning(m_ProcessId)) {
			m_ProcessId = 0;
		}
		return m_ProcessId?true:false;
	}
	return false;
}

const std::string &MediaHandler::getCurrentMediaName() const {
	return m_CurrentName;
}

const std::string &MediaHandler::getCurrentMediaPath() const {
	return m_Current;
}

bool MediaHandler::wereMultipleTracksPlaying() const {
	return m_bPlayingMultipleTracks;
}

bool MediaHandler::isMovie() const {
	return m_MediaType == Config::MEDIA_MOVIE;
}

bool MediaHandler::isMusic() const {
	return m_MediaType == Config::MEDIA_MUSIC;
}

