#include "smitten.hpp"
#include "fsprocess.hpp"

using namespace std;

MenuHandler::MenuHandler() {

	m_CurrentList = 0;
	m_CurrentScrollX = 0;
	m_CurrentScrollY = 0;
	
	m_PositionX = 0;
	m_PositionY = 0;

	m_Scale = 1/16.0f;
}

MenuHandler::~MenuHandler() {
	clearEntries();
}

int sgn(float n) {
	if (n < 0)	return -1;
	if (n > 0)	return 1;
	return 0;
}

void MenuHandler::update(float ) {
	int osx = sgn(m_PositionX);
	int osy = sgn(m_PositionY);
	m_PositionX += m_Scale * m_CurrentScrollX;
	m_PositionY += m_Scale * m_CurrentScrollY;

	int csx = sgn(m_PositionX);
	if (m_PositionX > 1 || m_PositionX < -1 || (osx!=csx && osx!=0)) {
		m_CurrentList += csx;
		m_PositionX = 0;
		m_CurrentScrollX = 0;
	}

	int csy = sgn(m_PositionY);
	if (m_PositionY > 1 || m_PositionY < -1 || (osy!=csy && osy!=0)) {
		m_EntryList[m_CurrentList]->m_CurrentEntry += csy;
		m_PositionY = 0;
		m_CurrentScrollY = 0;
	}
}

bool MenuItem::onSelect(MenuList *) {
	return true;	// this selection changes menu
}


void MenuHandler::populateMenu(unsigned int list, MenuItem *) {
	if (list >= m_EntryList.size()) {
		m_EntryList.reserve(list+1);
		int newEntries = (list+1) - m_EntryList.size();
		for(int i=0;i<newEntries;++i) {
			m_EntryList.push_back(new MenuList());
		}
	}

	if (m_EntryList[list]->m_List.size() > 0) {
		m_EntryList[list]->clear();
	}

}

void MenuList::clear() {
	vector<MenuItem *>::iterator it = m_List.begin();
	for(;it!=m_List.end();++it) {
		delete (*it);
	}
	
	m_List.clear();
}

void MenuHandler::clearEntries() {
	vector<MenuList *>::iterator it = m_EntryList.begin();
	for(;it!=m_EntryList.end();++it) {
		(*it)->clear();
		delete (*it);
	}
	
	m_EntryList.clear();
}

void MenuHandler::moveUp() {
	if (m_CurrentScrollX == 0) {
		if (m_EntryList[m_CurrentList]->m_CurrentEntry != 0 || m_CurrentScrollY == 1) {
			m_CurrentScrollY = -1;
		}
	}
}

void MenuHandler::moveDown() {
	MenuList *pMenuList = m_EntryList[m_CurrentList];
	if (m_CurrentScrollX == 0 && m_EntryList[m_CurrentList]->m_CurrentEntry+(m_CurrentScrollY==0?1:0) < pMenuList->m_List.size()) {
		m_CurrentScrollY = 1;
	}
}

void MenuHandler::moveStartList() {
	if (m_CurrentScrollX == 0) {
		MenuList *pMenuList = getCurrentMenuList();
		pMenuList->m_CurrentEntry = 0;
		m_CurrentScrollY = 0;
	}
}

void MenuHandler::moveEndList() {
	if (m_CurrentScrollX == 0) {
		MenuList *pMenuList = getCurrentMenuList();
		pMenuList->m_CurrentEntry = pMenuList->m_List.size() - 1;
		m_CurrentScrollY = 0;
	}
}


void MenuHandler::movePageDown(int bySoMany) {
	if (m_CurrentScrollX != 0) {
		return;
	}

	MenuList *pMenuList = m_EntryList[m_CurrentList];
	unsigned int highestEntry = pMenuList->m_List.size() - 1;
	unsigned int current = m_EntryList[m_CurrentList]->m_CurrentEntry;

	if (bySoMany < 0 && (unsigned int)-bySoMany > current) {
		current = 0;
	} else {
		current += bySoMany;
	}

	m_CurrentScrollY = 0;
	m_PositionY = 0;

	if (current < 0) {
		current = 0;
	} else if (current >= highestEntry) {
		current = highestEntry;
	}

	m_EntryList[m_CurrentList]->m_CurrentEntry = current;
}

void MenuHandler::movePageUp(int bySoMany) {
	movePageDown(-bySoMany);
}

// i.e. back
void MenuHandler::moveLeft() {
	if (m_CurrentScrollY == 0 && m_CurrentList != 0) {
		m_CurrentScrollX = -1;
	}
}

MenuItem *MenuHandler::getDirectedMenuItem(int depth) {
	if (depth == 0) {
		return NULL;
	}
	MenuList *pPreviousList = m_EntryList[depth-1];

	MenuItem *pItem = pPreviousList->m_List[pPreviousList->m_CurrentEntry];
	return pItem;
}

unsigned int MenuHandler::getCurrentMenuListIndex() {
	return m_CurrentList;
}

MenuList *MenuHandler::getCurrentMenuList() {
	return m_EntryList[m_CurrentList];
}

unsigned int MenuHandler::getCurrentMenuItemIndex() {
	MenuList *pMenuList = getCurrentMenuList();
	return pMenuList->m_CurrentEntry;
}

MenuItem *MenuHandler::getCurrentMenuItem() {
	MenuList *pMenuList = getCurrentMenuList();
	unsigned int curr = pMenuList->m_CurrentEntry;

	if (curr < 0 || curr >= pMenuList->m_List.size()) {
		return NULL;
	}

	MenuItem *pItem = pMenuList->m_List[curr];

	return pItem;
}

void MenuHandler::getCurrentAndSubsequentMenuItems(vector<MenuItem *> &items) {
	MenuList *pMenuList = getCurrentMenuList();
	unsigned int curr = pMenuList->m_CurrentEntry;

	if (curr < 0) {
		return;
	}

	while(curr < pMenuList->m_List.size()) {
		items.push_back(pMenuList->m_List[curr]);
		++curr;
	}
}


// i.e. make a selection
void MenuHandler::moveRight() {
	if (!isMoving()) {
		MenuItem *pItem = getCurrentMenuItem();

		if (pItem && pItem->onSelect(m_EntryList[m_CurrentList])) {
			// the onSelect for dir's will call populateNextMenu
		}
	}
}

void MenuHandler::select() {
	moveRight();
}

void MenuHandler::rootMenu() {
	populateMenu(0);
	m_CurrentList = 0;
	m_EntryList[0]->m_CurrentEntry = 0;
}

MenuList *MenuHandler::populateNextMenu() {
	populateMenu(m_CurrentList+1, NULL);
	
	m_EntryList[m_CurrentList+1]->m_CurrentEntry = 0;

	m_CurrentScrollX = 1;

	return m_EntryList[m_CurrentList+1];
}

bool  MenuHandler::isMoving() {
	if (m_CurrentScrollX == 0 && m_CurrentScrollY == 0) {
		return false;
	}
	return true;
}
