#include "smitten.hpp"
#include "SmittenUI.hpp"
#include "smittenskin.hpp"
#include "fsProcess.hpp"
#include "fsHandler.hpp"
#include "menutypes.hpp"
#include "sgxSystem.hpp"

MenuItem::MenuItem(std::string str) { 
	m_Name = str;
	m_Preview = NULL;

	const char *s = str.c_str();
	const char *p;
	p = strrchr(s, '/');
	if (p) {
		m_Name = p+1;
	} else {
		p = strrchr(s, '\\');
		if (p) {
			m_Name = p+1;
		}
	}

	// TODO Remove extensions
	// TODO Change . and _ to spaces
}

int MenuItem::getIconType() {
	return -1;
}

void MenuItem::getType(std::string &type) {
	type = "Unknown file type";
}

void MenuItem::getPreviewText(std::string &text) {
	text = "";
}

SmittenImage *MenuItem::getPreviewImage() {
	return m_Preview;
}

//
// Standard Menu : MediaDirectoryItem
//
MediaDirectoryItem::MediaDirectoryItem(const std::string &path) :  
MenuItem(path),  m_Filepath(path) {
}

MediaDirectoryItem::MediaDirectoryItem(const std::string &name, const std::string &path) :  
MenuItem(name),  m_Filepath(path) {
}

int MediaDirectoryItem::getIconType() {
	return SmittenSkin::UI_ICON_DIR;
}


bool MediaDirectoryItem::onSelect(MenuList *) {

	std::vector<fsEntry *> directory;
	fsProcess::getDirectory(directory, m_Filepath);

	// TODO: utilize sgxBilingual

	MenuList *pMenuList = SmittenUI::get()->m_pMenu->populateNextMenu();
	pMenuList->clear();
	pMenuList->m_List.push_back((MenuItem *)new GoBackFileItem(std::string("Back")));
	fsHandler::buildMenu(pMenuList->m_List, directory);

	// Don't start the menu on the 'back' button, unless it's the only one there
	if (pMenuList->m_List.size() > 1) {
		pMenuList->m_CurrentEntry = 1;
	}
	return false;
}

//
// Standard Menu : GoBackFileItem
//
bool GoBackFileItem::onSelect(MenuList *) {
	SmittenUI::get()->onAction(SmittenUI::ActionGoBack);
	return false;
}

int GoBackFileItem::getIconType() {
	return SmittenSkin::UI_ICON_BACK;
}


//
// Standard Menu : ExitSmittenItem
//
ExitSmittenItem::ExitSmittenItem(int phase) : MenuItem("") {
	m_Phase = phase;
	switch(phase) {
		case 0: m_Name = "Exit?"; break;
		case 1: m_Name = "Yes - I'm sure"; break;
		case 2: m_Name = "No"; break;
	}
}

bool ExitSmittenItem::onSelect(MenuList *) {

	if (m_Phase == 0) {
		MenuList *pMenuList = SmittenUI::get()->m_pMenu->populateNextMenu();

		pMenuList->clear();
		pMenuList->m_List.push_back((MenuItem *)new ExitSmittenItem(1));
		pMenuList->m_List.push_back((MenuItem *)new ExitSmittenItem(2));
	
	} else if (m_Phase == 1) {
		SmittenUI::get()->onAction(SmittenUI::ActionQuit);

	} else {
		SmittenUI::get()->m_pMenu->moveLeft();
	}
	return true;
}
