/*
    Smitten: A media centre application, for playing movies, music, picture
             slideshows, and reading text on your TV or computer.

    Copyright (C) 2008 Steven Goodwin

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; 
    version 2.0.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
#include "SDL/SDL.h" 
#include "smitten.hpp"
#include "SmittenUI.hpp"
#include "SmittenMenuHandler.hpp"
#include "menutypes.hpp"
#include "fsProcess.hpp"
#include "fsHandler.hpp"

void SmittenInit(SDL_Surface *pScreen) {
	SmittenUI::get()->create();
	SmittenUI::get()->setScreenImage(SmittenImage::create(pScreen));

	if (!SmittenUI::get()->m_pSkin->wantMousePointer()) {
		SDL_ShowCursor(0);
	}
}

bool SmittenUpdate(SDL_Event *pEvent, int telapsMilliSeconds) {
	static bool autorepeatLocked[SDLK_LAST];

	SmittenUI::get()->m_bIsDirty = false;

	if (pEvent->type == SDL_KEYDOWN && !autorepeatLocked[pEvent->key.keysym.sym]) {
		switch(pEvent->key.keysym.sym) {
			case SDLK_f:
			case SDLK_UP:
				SmittenUI::get()->onAction(SmittenUI::ActionPrevious);
				break;
			case SDLK_v:
			case SDLK_DOWN:
				SmittenUI::get()->onAction(SmittenUI::ActionNext);
				break;
			case SDLK_BACKSPACE:
			case SDLK_LEFT:
				SmittenUI::get()->onAction(SmittenUI::ActionGoBack);
				break;
			case SDLK_RIGHT:
				SmittenUI::get()->onAction(SmittenUI::ActionGoForward);
				break;
			case SDLK_PAGEDOWN:
				SmittenUI::get()->onAction(SmittenUI::ActionSkipForward);	
				break;
			case SDLK_PAGEUP:
				SmittenUI::get()->onAction(SmittenUI::ActionSkipBackwards);	
				break;
			case SDLK_HOME:
				SmittenUI::get()->onAction(SmittenUI::ActionSkipStart);	
				break;
			case SDLK_END:
				SmittenUI::get()->onAction(SmittenUI::ActionSkipEnd);	
				break;

			case SDLK_RETURN:
				SmittenUI::get()->onAction(SmittenUI::ActionSelect);
				break;
			case SDLK_ESCAPE:
				SmittenUI::get()->onAction(SmittenUI::ActionRootMenu);
				break;
			case SDLK_q:
				SmittenUI::get()->onAction(SmittenUI::ActionQuit);
				return false;
		}
		//
		switch(pEvent->key.keysym.sym) {
			case SDLK_PAGEUP:
			case SDLK_PAGEDOWN:
			case SDLK_LEFT:
			case SDLK_RIGHT:
			case SDLK_RETURN:
				autorepeatLocked[pEvent->key.keysym.sym] = true;
				break;
		}

	} else if (pEvent->type == SDL_KEYUP) {
		switch(pEvent->key.keysym.sym) {
			case SDLK_PAGEUP:
			case SDLK_PAGEDOWN:
			case SDLK_LEFT:
			case SDLK_RIGHT:
			case SDLK_RETURN:
				autorepeatLocked[pEvent->key.keysym.sym] = false;
				break;
		}
	}
		
	return SmittenUI::get()->update(telapsMilliSeconds/1000.0f);
}

bool SmittenPaint() {
	return SmittenUI::get()->draw();
}

