#include "sdl/sdl.h"
#include "smitten.hpp"
#include "smittenUI.hpp"
#include "menutypes.hpp"

SmittenFont::SmittenFont(const std::string& filename, const std::string& glyphs) {

	m_RowSpacing = 0;
	m_GlyphSpacing = 0;
	m_FontHeight = 0;

	for(int i=0;i<MAX_GLYPHS;++i) {
		m_pGlyphOffsets[i][0] = m_pGlyphOffsets[i][1] = 0;
	}

	m_pFontImage = SmittenImage::create(filename);
	if (m_pFontImage) {
		m_FontHeight = m_pFontImage->getHeight();

		unsigned int separate = m_pFontImage->getColor(0,0);
		unsigned int col;

		m_pFontImage->lockPixels();
		int cx = 0;
		int px = 0;

		// skip separater
		col = separate;
		while(col == separate) {
			++cx;
			col = m_pFontImage->getColor(cx, 0);
		}

		// now grab each glyph
		for(unsigned int g=0;g<glyphs.size();++g) {
			px = cx;
			do {
				++cx;
				col = m_pFontImage->getColor(cx, 0);
			} while (col != separate);

			// got the glyph
			m_pGlyphOffsets[glyphs[g]][0] = px;
			m_pGlyphOffsets[glyphs[g]][1] = cx;

			// now skip the separator
			while(col == separate) {
				++cx;
				col = m_pFontImage->getColor(cx, 0);
			}

		}

		m_pFontImage->unlockPixels();
	}

}

int SmittenFont::getWidth(const std::string &str) const {
	int w = 0;
	for(unsigned int i=0;i<str.size();++i) {
		w += getWidth(str[i]);
		w += m_GlyphSpacing;
	}
	return w;
}

int SmittenFont::getWidth(const unsigned char c)  const {
	if (c >= MAX_GLYPHS) {
		return 0;
	}
	return m_pGlyphOffsets[c][1] - m_pGlyphOffsets[c][0];
}

int SmittenFont::getHeight() const {
	return m_FontHeight;
}

int SmittenFont::drawString(const std::string &str, int x, int y) const {
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	int ox = x;
	for(unsigned int i=0;i<str.size();++i) {
		const unsigned char &c = str[i];

		// Check for special characters
		if (c >= MAX_GLYPHS) {
			// NULL
		} else if (c == '\n') {
			x = ox;
			y += getHeight();
			y += m_RowSpacing;
		} else {
			m_pFontImage->drawArea(pScreenImage, x, y, m_pGlyphOffsets[c][0], 0, m_pGlyphOffsets[c][1], m_FontHeight);
			x += getWidth(c);
			x += m_GlyphSpacing;
		}
	}

	return 0;
}

