#include "smitten.hpp"
#include "smittenUI.hpp"
#include "menutypes.hpp"

#define LINE_SPACING	2
#define LINE_GAP		8
#define LINE_SIZE		32
#define LINE_HEIGHT		(LINE_SIZE+LINE_GAP)

using namespace std;

SmittenSkin::SmittenSkin(const std::string &visualizer) {
	m_MenuArea = sRect(10,87,469-10,581-87);
	m_PreviewArea = sRect(482,87,320,200);
	m_StatusArea = sRect(482,560,630,20);
	m_DirectoryArea = sRect(10,45,0,0);

	m_Visualizer = visualizer;
// TODO: Preview area allows normal code to stretchblit info into it
}

std::string &SmittenSkin::getSkinName() {
	static std::string n("std");
	return n;
}


void SmittenSkin::create() {
	loadImageSet();
}

void SmittenSkin::loadImageSet() {
	std::string path = getPath();

	m_MenuListFont = new SmittenFont(path+"menulist.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`\'*#=[]\"");
	m_StatusFont = m_PreviewFont = m_TextBodyFont = m_MenuListFont;

	m_pBackground = getImage("bg.png");
	m_pBar = getImage("bar.png");
	m_pDirDivider = getImage("div.png");

	m_pIcons[UI_ICON_BACK] = getImage("back.png");
	m_pIcons[UI_ICON_TEXT] = getImage("icontxt.png");
	m_pIcons[UI_ICON_MOVIE] = getImage("iconvid.png");
	m_pIcons[UI_ICON_IMAGE] = getImage("iconpix.png");
	m_pIcons[UI_ICON_DIR] = getImage("icondir.png");
	m_pIcons[UI_ICON_MUSIC] = getImage("iconmus.png");
}

std::string SmittenSkin::getPath() {
	std::string path("skins");
	path += "/";
	path +=	getSkinName();
	path += "/";
	path += m_Visualizer;
	path += "/";
	return path;
}

SmittenImage *SmittenSkin::getImage(const std::string &name) {

	std::string path = getPath();
	path += name;
	return SmittenImage::create(path);
}

int SmittenSkin::getMenuPageSize() {
	return m_MenuArea.h / LINE_HEIGHT;
}

int SmittenSkin::getTextPageSize() {
	return getHeight() / (m_TextBodyFont->getHeight() + LINE_SPACING);
}

int SmittenSkin::getWidth() {
	return 800;
}

int SmittenSkin::getHeight() {
	return 600;
}

int SmittenSkin::getPreviewImageWidth() {
	return 320;
}

int SmittenSkin::getPreviewImageHeight() {
	return 200;
}

bool SmittenSkin::wantMousePointer() {
	return false;
}

void SmittenSkin::draw(MenuHandler *pMenu) {
	// clear screen in some way
	if (m_pBackground) {
		m_pBackground->draw(SmittenUI::get()->getScreenImage(), 0, 0);
	} else {
		SmittenUI::get()->getScreenImage()->clear(0);
	}

	int visibleItems = m_MenuArea.h / LINE_HEIGHT;
	int totalItems = pMenu->m_EntryList[pMenu->m_CurrentList]->m_List.size();

	if (totalItems > visibleItems) {
		drawCentreSelection(pMenu);
	} else {
		drawStatic(pMenu);
	}

	// preview
	drawPreview(pMenu);

	// status
	drawStatusInformation(pMenu);

	drawDirectoryPath(pMenu);

	drawAlways(pMenu);
}

void SmittenSkin::drawAlways(MenuHandler *) {
}

bool SmittenSkin::update(float) {
	return false;
}

void SmittenSkin::drawDirectoryPath(MenuHandler *pMenu) {
	int x = m_DirectoryArea.x;
	int y = m_DirectoryArea.y + (LINE_HEIGHT-m_MenuListFont->getHeight())/2;

	for(unsigned int i=1;i<=pMenu->getCurrentMenuListIndex();++i) {
		string txt = pMenu->getDirectedMenuItem(i)->visibleName();
		m_MenuListFont->drawString(txt, x, y);
		x += m_MenuListFont->getWidth(txt);
		if (m_pDirDivider) {
			m_pDirDivider->draw(SmittenUI::get()->getScreenImage(), x, m_DirectoryArea.y);
			x += m_pDirDivider->getWidth();
		} else {
			x += 20;
		}
	}
}

void SmittenSkin::drawPreview(MenuHandler *pMenu) {
	MenuItem *pCurrent = pMenu->getCurrentMenuItem();
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	if (pCurrent) {
		int y = m_PreviewArea.y;

		pScreenImage->clip();
		m_StatusFont->drawString(pCurrent->visibleName(), m_PreviewArea.x, y);

		SmittenImage *img = pCurrent->getPreviewImage();
		
		y += m_StatusFont->getHeight();
		if (img) {
			img->draw(SmittenUI::get()->getScreenImage(), m_PreviewArea.x, y);
			y += m_PreviewArea.h + LINE_GAP;
		}

		string txt;
		pCurrent->getPreviewText(txt);
		m_StatusFont->drawString(txt, m_PreviewArea.x, y);

	}
}

void SmittenSkin::drawStatusInformation(MenuHandler *) {
	MediaHandler *p = SmittenUI::get()->m_pMedia;
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	pScreenImage->clip();

	if (p->isPlaying()) {
		m_StatusFont->drawString("Playing...", m_StatusArea.x, m_StatusArea.y-m_StatusFont->getHeight()-LINE_GAP);
		m_StatusFont->drawString(p->getCurrentMediaName().c_str(), m_StatusArea.x, m_StatusArea.y);
	} else {
		m_StatusFont->drawString("Ready...", m_StatusArea.x, m_StatusArea.y);
	}
}

void SmittenSkin::drawCentreSelection(MenuHandler *pMenu) {
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	// anything in here will need to appear/be visible to pSkin
	MenuList *pMenuList = pMenu->getCurrentMenuList();

	int border=0;
	pScreenImage->clip(m_MenuArea.x+border, m_MenuArea.y+border, m_MenuArea.w-border, m_MenuArea.h-border);

	int gap = LINE_GAP;
	int offset = gap + LINE_SIZE;
	int centreLine = m_MenuArea.y + m_MenuArea.h/2 - m_MenuListFont->getHeight()/2;

	// we always have 2 extra items, so we're drawing the ones just off the top,
	// and just off the bottom, when scrolling
	int totalItems = pMenuList->m_List.size();

	int margin = 4;
	int x = m_MenuArea.x+5 - (int)(pMenu->m_PositionX*m_MenuArea.w);
	int y = (centreLine - pMenu->getCurrentMenuItemIndex() * offset) - (int)(pMenu->m_PositionY*offset);

	sRect dr(x-margin, centreLine, m_MenuArea.w-margin, LINE_SIZE);
	if (m_pBar) {
		m_pBar->draw(pScreenImage, dr.x, dr.y);
	} else {
		pScreenImage->fillRect(dr, 51,128,255);
	}

	for(int i=0;i<totalItems;++i) {
		// rely on the ClipRect to prune off-screen items
		drawMenuItem(pMenuList->m_List[i], x, y);
		y += offset;
	}
}

void SmittenSkin::drawStatic(MenuHandler *pMenu) {
	// anything in here will need to appear/be visible to pSkin
	MenuList *pMenuList = pMenu->m_EntryList[pMenu->m_CurrentList];
	SmittenImage *pScreenImage = SmittenUI::get()->getScreenImage();

	int border = 4;
	sRect r(m_MenuArea.x+border, m_MenuArea.y+border, m_MenuArea.w-border, m_MenuArea.h-border);

	if (pMenuList->m_List.size() == 0) {
		return;
	}

	vector<MenuItem *>::iterator it = pMenuList->m_List.begin();

	int margin=LINE_GAP;
	int gap = LINE_GAP;
	int offset = gap + LINE_SIZE;

	int oy = m_MenuArea.y + (m_MenuArea.h-(pMenuList->m_List.size()*offset - gap))/2;
	int x = 10 + m_MenuArea.x - (int)(pMenu->m_PositionX*m_MenuArea.w);
	int y = oy;
	int idx = 0;

	pScreenImage->clip(r);
	
	if (pMenu->m_PositionX == 0) {
		sRect dr(x-margin, oy+pMenu->getCurrentMenuItemIndex()*offset+(int)(pMenu->m_PositionY*offset),m_MenuArea.w-margin,LINE_SIZE);

		if (m_pBar) {
			m_pBar->draw(pScreenImage, dr.x, dr.y-(m_pBar->getHeight()-LINE_SIZE)/2);
		} else {
			pScreenImage->fillRect(dr, 51,128,255);
		}
	}

	for(;it != pMenuList->m_List.end();++it,++idx) {
		drawMenuItem(*it, x, y);
		y += offset;
	}

	pScreenImage->clip();
}


void SmittenSkin::drawMenuItem(MenuItem *pItem, int x, int y) {
	sRect dr (x,y,32,32);

	int type = pItem->getIconType();

	if (type < 0 || type >= sizeof(m_pIcons)/sizeof(m_pIcons[0])) {
		SmittenUI::get()->getScreenImage()->fillRect(dr, 0,0,0);
	} else if (m_pIcons[type]) {
		m_pIcons[type]->draw(SmittenUI::get()->getScreenImage(), dr.x, dr.y);
	}

	m_MenuListFont->drawString(pItem->visibleName(), x+42, y+((LINE_SIZE+LINE_GAP)-m_MenuListFont->getHeight())/2);

}

void SmittenSkin::drawTextFile(sgxTextFile *pText, int line, int , int x, int y) {
	int h = SmittenUI::get()->getScreenImage()->getHeight();
	while(y<h) {
		m_TextBodyFont->drawString(pText->getLine(line), x, y);
		++line;
		y += m_TextBodyFont->getHeight();
		y += LINE_SPACING;
	}
}



